@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8:
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,11 +18,11 @@
 from __future__ import absolute_import
 
 import errno
-import gobject
 import locale
 import logging
 import os
 import shutil
+import sqlite3
 import subprocess
 import sys
 import tempfile
@@ -44,6 +44,29 @@ from solfege import utils
 
 import solfege
 
+try:
+    from pyalsa import alsaseq
+except ImportError:
+    alsaseq = None
+
+solfege_copyright = u"Copyright © 1999-2008 Tom Cato Amundsen <tca@gnu.org>, and others."
+
+warranty = """
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+"""
+
+
 def check_rcfile():
     """See default.config for rcfileversion values, meanings and
     a description of how to add config variables.
@@ -54,7 +77,7 @@ def check_rcfile():
         return
     if cfg.get_int("app/rcfileversion") <= 1:
         if not "example-files" in cfg.get_string('config/lessoncollections'):
-            cfg.set_string('config/lessoncollections', 
+            cfg.set_string('config/lessoncollections',
                 "%s example-files" % cfg.get_string('config/lessoncollections'))
     if cfg.get_int("app/rcfileversion") <= 5:
         # This is more complicated that necessary to fix an old
@@ -121,7 +144,7 @@ def check_rcfile():
     if cfg.get_int("app/rcfileversion") < 17:
         v = cfg.get_string("app/frontpage").split("/")
         if v[0] == u"exercises" and v[1] != u"standard":
-            cfg.set_string("app/frontpage", 
+            cfg.set_string("app/frontpage",
                            u"/".join([v[0], u"standard"] + v[1:]))
     if cfg.get_int("app/rcfileversion") < 18:
         cfg.del_key("gui/web_browser_as_help_browser")
@@ -173,6 +196,29 @@ class SolfegeApp(cfg.ConfigUtils):
         if self.m_options.no_sound \
            or cfg.get_string("sound/type") == "fake-synth":
             soundcard.initialise_using_fake_synth(self.m_options.verbose_sound_init)
+        elif cfg.get_string("sound/type") == "alsa-sequencer":
+            if alsaseq:
+                try:
+                    clientid, portid = self.get_list("sound/alsa-client-port")
+                except ValueError:
+                    clientid, portid = (None, None)
+                try:
+                    soundcard.initialise_alsa_sequencer((clientid, portid),
+                            self.m_options.verbose_sound_init)
+                except alsaseq.SequencerError, e:
+                    logging.debug("initialise_alsa_sequencer failed. Using fake synth.")
+                    self.display_sound_init_error_message(e)
+                    soundcard.initialise_using_fake_synth(True)
+                    return
+            else:
+                if solfege.splash_win:
+                    solfege.splash_win.hide()
+                gu.dialog_ok(_("The pyalsa Python module is missing"),
+                    solfege.win,
+                    _("Solfege was configured to use the Python modules from www.alsa-project.org, but the modules were not found. You must reconfigure sound in the preferences window (Ctrl-F12) or restart Solfege in a way that it finds the modules."))
+                soundcard.initialise_using_fake_synth(True)
+                if solfege.splash_win:
+                    solfege.splash_win.show()
         elif cfg.get_string("sound/type") == "winsynth":
             try:
                 soundcard.initialise_winsynth(cfg.get_int("sound/synth_number"),
@@ -195,7 +241,6 @@ class SolfegeApp(cfg.ConfigUtils):
                 cfg.set_int("sound/synth_number", soundcard.synth.m_devnum)
         elif cfg.get_string("sound/type") == "external-midiplayer":
             soundcard.initialise_external_midiplayer(
-                    cfg.get_string("sound/midi_player"),
                     verbose_init=self.m_options.verbose_sound_init)
             soundcard.synth.error_report_cb = solfege.win.display_error_message
         elif cfg.get_string("sound/type") == '':
@@ -233,13 +278,13 @@ class SolfegeApp(cfg.ConfigUtils):
             """%s""" % str(e).decode(locale.getpreferredencoding(), 'replace'))
         elif isinstance(e, ImportError):
             solfege.win.display_error_message2(str(e), _("You should configure sound from the preferences window, and try to use an external midi player. Or try to recompile the program and check for error messages to see why the module is not built."))
-        elif e.errno == errno.EACCES:
+        elif getattr(e, 'errno', None) == errno.EACCES:
             solfege.win.display_error_message(
                 "The sound init failed: %s\n"
                 "The errno EACCES indicates that you don't have write "
                 "permission to the device."
                 % str(e).decode(locale.getpreferredencoding(), 'replace'))
-        elif e.errno == errno.EBUSY:
+        elif getattr(e, 'errno', None) == errno.EBUSY:
             solfege.win.display_error_message(
                 "The sound init failed: %s\n"
                 "It seems like some other program is using the device. You "
@@ -270,7 +315,7 @@ class SolfegeApp(cfg.ConfigUtils):
                 self.handle_href('%s.html' % solfege.win.m_viewer)
     def show_exercise_theory(self):
         if self.m_teachers[self.m_running_exercise].m_P.header.theory:
-            solfege.win.display_docfile("%s.html" % self.m_teachers[self.m_running_exercise].m_P.header.theory, "")
+            solfege.win.display_docfile("%s.html" % self.m_teachers[self.m_running_exercise].m_P.header.theory)
     def _practise_lessonfile(self, filename, urlobj=None):
         """
         return the module name.
@@ -355,8 +400,14 @@ class SolfegeApp(cfg.ConfigUtils):
             # exceptions.
             if g.m_t.m_P and getattr(g.m_t.m_P, 'header', None):
                 g.on_end_practise()
-        solfege.db.conn.commit()
-        solfege.db.conn.close()
+        try:
+            solfege.db.conn.commit()
+        except sqlite3.ProgrammingError, e:
+            gu.display_exception_message(e)
+        try:
+            solfege.db.conn.close()
+        except sqlite3.ProgrammingError, e:
+            gu.display_exception_message(e)
         if soundcard.synth:
             soundcard.synth.close()
         shutil.rmtree(lessonfile.MusicBaseClass.temp_dir, True)
@@ -435,7 +486,7 @@ class SolfegeApp(cfg.ConfigUtils):
                                     (self.get_int('config/default_bpm'), 4))
                     soundcard.end_export()
                 else:
-                    logging.warning("export_training_set:ignoring exercise with module='%s'" % module)
+                    logging.warning("export_training_set:ignoring exercise with module='%s'", module)
 #####
                 def do_convert(from_format, to_format):
                     """
