@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2009, 2010  Tom Cato Amundsen
+# Copyright (C) 2009, 2010, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +17,11 @@
 
 from __future__ import absolute_import
 
+import locale
 import logging
+import sys
 
+import gobject
 import gtk
 
 import solfege
@@ -26,6 +29,12 @@ from solfege import frontpage
 from solfege import gu
 from solfege import lessonfile
 
+def decode_entry_string(s):
+    if sys.platform == 'win32':
+        return s.decode("mbcs", "replace")
+    return s.decode(locale.getpreferredencoding(), "decode")
+
+
 class SelectWinBase(gtk.ScrolledWindow):
     """
     Base class for the classes that display the front page and the
@@ -121,7 +130,7 @@ class SelectWinBase(gtk.ScrolledWindow):
         # ubuntu 10.04 and +4 on MS Windows XP.
         h = self.g_box.size_request()[1] + 4
         if h > gtk.gdk.screen_height() * 0.8:
-            h = gtk.gdk.screen_height() *0.8
+            h = int(gtk.gdk.screen_height() *0.8)
         self.set_size_request(w, h if h > self.m_min_height else self.m_min_height)
         # Then we check if we have to move the app window higher up on the
         # screen so that the whole window is visible.
@@ -271,8 +280,10 @@ class ExerciseView(SelectWinBase):
         self.get_vadjustment().set_value(0.0)
     def on_page_link_clicked(self, btn, link):
         self.g_searchbox.hide()
+        self.emit('link-clicked', self.m_page)
         self.display_data(link)
     def on_link_clicked(self, w, filename):
+        self.emit('link-clicked', self.m_page)
         solfege.app.practise_lessonfile(filename)
     def display_search_result(self, searchfor, result, result_C, display_only_tests=False):
         self._display_data(
@@ -324,6 +335,11 @@ class ExerciseView(SelectWinBase):
                     pass
         return page
 
+gobject.signal_new('link-clicked', ExerciseView,
+                   gobject.SIGNAL_RUN_FIRST,
+                   gobject.TYPE_NONE,
+                   (gobject.TYPE_PYOBJECT,))
+
 
 class FrontPage(ExerciseView):
     def display_data(self, data, display_only_tests=False,
@@ -332,8 +348,8 @@ class FrontPage(ExerciseView):
                            show_topics)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
-        logging.debug("FrontPage.on_search '%s'", search_for)
+        search_for = decode_entry_string(self.g_searchentry.get_text().lower())
+        logging.debug(u"FrontPage.on_search '%s'", search_for)
         page = self._search(search_for, False, False)
         page_C = self._search(search_for, True, False)
         page_C.m_name = _('Search untranslated lesson titles')
@@ -345,12 +361,13 @@ class FrontPage(ExerciseView):
 
 class TestsView(ExerciseView):
     def on_link_clicked(self, w, filename):
+        self.emit('link-clicked', self.m_page)
         solfege.app.test_lessonfile(filename)
     def display_data(self, data=None, is_search_result=False, show_topics=False):
         self._display_data(data, True, is_search_result, show_topics)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
+        search_for = decode_entry_string(self.g_searchentry.get_text().lower())
         page = self._search(search_for, False, True)
         page_C = self._search(search_for, True, True)
         page_C.m_name = u'Search exercises without translating them'
@@ -377,9 +394,9 @@ class SearchView(ExerciseView):
         self._display_data(data)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
+        search_for = decode_entry_string(self.g_searchentry.get_text().lower())
         lessonfile.infocache.update_modified_files()
-        self.display_search_result(self.g_searchentry.get_text(), [
+        self.display_search_result(decode_entry_string(self.g_searchentry.get_text()), [
             filename for filename in lessonfile.infocache._data
             if search_for in _(lessonfile.infocache.get(filename, 'title')).lower()
         ], [
