@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -115,6 +115,70 @@ def random_tonika_and_interval(lowest, highest, irange):
     return tonika, interval
 
 
+def random_interval_in_key(first, lowest, highest, irange, tonic, keytype):
+    """
+    Return an int representing the interval.
+    Return None if it is not possible to create an interval.
+    Arguments:
+        first   The MusicalPitch of the first tone
+        lowest
+        highest integer representing the MIDI pitch of the tone
+        tonic   MusicalPitch, the key the tones should be taken from
+        keytype "major", "natural-minor" or "harmonic-minor"
+    """
+    assert isinstance(lowest, int)
+    assert isinstance(highest, int)
+    assert isinstance(first, mpd.MusicalPitch)
+    tones = pitches_in_key(tonic, keytype, lowest, highest)
+    intervals = []
+    for i in irange:
+        if (first + i).semitone_pitch() in tones:
+            intervals.append(i)
+    if intervals:
+        return random.choice(intervals)
+    
+
+def random_tonic_and_interval_in_key(lowest, highest, irange, tonic, keytype):
+    """
+    Return a tuple (MusicalPitch, int) where the musical pitch is the
+    lowest tone and the integer is the size of the interval.
+    Arguments:
+        lowest
+        highest integer representing the MIDI pitch of the tone
+        tonika  MusicalPitch, the key the tones should be taken from
+        keytype "major", "natural-minor" or "harmonic-minor"
+    """
+    # We allow str because it simplifies writing the tests
+    if isinstance(tonic, str):
+        tonic = mpd.MusicalPitch.new_from_notename(tonic)
+    # Make a list of all tones in the range that belongs to the key.
+    tones = pitches_in_key(tonic, keytype, lowest, highest)
+    interval = random.choice(irange)
+    # possible lowest tones
+    plow = []
+    for tone in tones:
+        if tone + interval in tones:
+            plow.append(tone)
+    return (mpd.MusicalPitch.new_from_int(random.choice(plow)), interval)
+
+random_tonic_and_interval_in_key.key_data = {'major': (0, 2, 4, 5, 7, 9, 11),
+         'natural-minor': (0, 2, 3, 5, 7, 8, 10),
+         'harmonic-minor': (0, 2, 3, 5, 7, 8, 10),
+    }
+
+def pitches_in_key(tonic, keytype, lowest, highest):
+    """
+    Return a set of the pitches (int values corresponding to
+    the MIDI spec and MusicalPitch.semitone_pitch())
+    """
+    tones = set()
+    p = tonic.pitch_class()
+    for octave in range(-1, 12):
+        for t in random_tonic_and_interval_in_key.key_data[keytype]:
+            if lowest <= p + octave * 12 + t <= highest:
+                tones.add(p + octave * 12 + t)
+    return tones
+
 def adjust_low_high_to_irange(lowest, highest, irange):
     """
     lowest
